/*
 * Copyright (c) 2021, 2023, 2025 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *					Anthony Mallet on Tue Jul 27 2021
 */
#ifndef H_GAZEBOFT_CODELS
#define H_GAZEBOFT_CODELS

#include "gazeboft_c_types.h"

#include <aio.h>
#include <errno.h>
#include <inttypes.h>
#include <stdbool.h>
#include <string.h>

#include <condition_variable>
#include <mutex>

#include "gz/msgs.hh"
#include "gz/transport.hh"

struct gazeboft_topic {
  void cb(const gz::msgs::Wrench &msg);

  mutable std::mutex m{};
  mutable std::condition_variable cv{};
  mutable bool fresh{false};
  or_wrench_estimator_state w;

  gz::transport::Node *node{nullptr};
};


struct gazeboft_log_s {
  struct aiocb req;
  char buffer[4096];
  bool pending, skipped;
  uint32_t decimation;
  size_t missed, total;

# define gazeboft_log_header	"ts  fx fy fz  tx ty tz"
# define gazeboft_log_line	"%" PRIu64 ".%09d  %g %g %g  %g %g %g"
};

static inline genom_event
gazeboft_e_sys_error(const char *s, genom_context self)
{
  gazeboft_e_sys_detail d;
  size_t l = 0;

  d.code = errno;
  if (s) {
    strncpy(d.what, s, sizeof(d.what) - 3);
    l = strlen(s);
    strcpy(d.what + l, ": ");
    l += 2;
  }
  snprintf(d.what + l, sizeof(d.what) - l, "%s", strerror(d.code));
  return gazeboft_e_sys(&d, self);
}

#endif /* H_GAZEBOFT_CODELS */
